@@ -14,12 +14,15 @@ from astrocats.catalog.entry import ENTRY, Entry
 from astrocats.catalog.key import KEY_TYPES, Key
 from astrocats.catalog.photometry import (PHOTOMETRY, set_pd_mag_from_counts,
                                           set_pd_mag_from_flux_density)
+from astrocats.catalog.quantity import QUANTITY
 from astrocats.catalog.source import SOURCE
 from astrocats.catalog.utils import jd_to_mjd
 from astropy.io.ascii import Cds, Latex, read
 from astropy.time import Time as astrotime
-from mosfit.utils import (entabbed_json_dump, is_date, is_number, listify,
-                          name_clean)
+from mosfit.constants import KS_DAYS
+from mosfit.utils import (entabbed_json_dump, get_mosfit_hash, is_bibcode,
+                          is_date, is_datum, is_number, listify, name_clean,
+                          replace_multiple)
 from six import string_types
 
 
@@ -56,53 +59,97 @@ class Converter(object):
 
     _TRUE_VALS = ['t', 'true', 'T', 'True', '1', 'y', 'Y']
     _FALSE_VALS = ['f', 'false', 'F', 'False', '0', 'n', 'N']
+    _EMPTY_VALS = ['nodata']
 
     def __init__(self, printer, require_source=False, **kwargs):
         """Initialize."""
+        import pickle
+
+        self._path = os.path.dirname(os.path.realpath(__file__))
         self._inflect = inflect.engine()
         self._printer = printer
         self._require_source = require_source
 
-        self._emagstrs = [
-            'magnitude error', 'error', 'e mag', 'e magnitude', 'dmag',
-            'mag err', 'magerr', 'mag error', 'err', '_err', 'err_', 'ERR']
+        self._estrs = [
+            'err', '_err', 'err_', 'ERR', 'e_', '_e', '(err)', 'error',
+            'uncertainty', 'sigma']
+        self._emagstrs = self._estrs + [
+            'magnitude error', 'e mag', 'e magnitude', 'dmag',
+            'mag err', 'magerr', 'mag error']
+        self._ecntstrs = self._estrs + [
+            'flux error', 'e flux', 'e counts', 'count err', 'flux err',
+            'countrate error', 'countrate err', 'e_flux']
         self._band_names = [
-            'U', 'B', 'V', 'R', 'I', 'J', 'H', 'K', 'K_s', "K'", 'u', 'g',
-            'r', 'i', 'z', 'y', 'W1', 'W2', 'M2', "u'", "g'", "r'", "i'",
-            "z'", 'C', 'Y'
+            'U', 'B', 'V', 'R', 'I', 'J', 'H', 'K', 'K_s', "Ks", "K'", 'u',
+            'g', 'r', 'i', 'z', 'y', 'W1', 'W2', 'M2', "u'", "g'", "r'", "i'",
+            "z'", 'C', 'Y', 'Open'
         ]
-        self._emagstrs = self._emagstrs + [a + b for a, b in chain(
-            product(self._emagstrs, self._band_names),
-            product(self._band_names, self._emagstrs))]
-        self._header_keys = OrderedDict((
-            (PHOTOMETRY.TIME, ['time', 'mjd', ('jd', 'jd'), 'date']),
-            (PHOTOMETRY.SYSTEM, ['system']),
-            (PHOTOMETRY.MAGNITUDE, ['vega mag', 'ab mag', 'mag', 'magnitude']),
-            (PHOTOMETRY.E_MAGNITUDE, self._emagstrs),
-            (PHOTOMETRY.TELESCOPE, ['tel', 'telescope']),
-            (PHOTOMETRY.INSTRUMENT, ['inst', 'instrument']),
-            (PHOTOMETRY.BAND, ['passband', 'band', 'filter', 'flt']),
-            (PHOTOMETRY.E_LOWER_MAGNITUDE, [a + ' ' + b for a, b in chain(
-                product(self._emagstrs, ['minus', 'lower']),
-                product(['minus', 'lower'], self._emagstrs))]),
-            (PHOTOMETRY.E_UPPER_MAGNITUDE, [a + ' ' + b for a, b in chain(
-                product(self._emagstrs, ['plus', 'upper']),
-                product(['plus', 'upper'], self._emagstrs))]),
-            (PHOTOMETRY.UPPER_LIMIT, [
-             'upper limit', 'upperlimit', 'l_mag', 'limit']),
-            (PHOTOMETRY.COUNT_RATE, ['counts', 'flux', 'count rate']),
-            (PHOTOMETRY.E_COUNT_RATE, [
-                'e_counts', 'count error', 'count rate error']),
-            (PHOTOMETRY.FLUX_DENSITY, ['flux density', 'fd', 'f_nu']),
-            (PHOTOMETRY.E_FLUX_DENSITY, [
-                'e_flux_density', 'flux density error', 'e_fd', 'sigma_nu']),
-            (PHOTOMETRY.U_FLUX_DENSITY, []),
-            (PHOTOMETRY.ZERO_POINT, ['zero point', 'zp']),
-            ('reference', ['reference', 'bibcode', 'source', 'origin']),
-            (ENTRY.NAME, [
-                'event', 'transient', 'name', 'supernova', 'sne', 'id',
-                'identifier'])
-        ))
+        ebands = [a + b for a, b in chain(
+            product(self._ecntstrs, self._band_names),
+            product(self._band_names, self._estrs))]
+        self._emagstrs += ebands
+        self._ecntstrs += ebands
+        key_cache_path = os.path.join(
+            self._path, 'cache', 'key_cache_{}.pickle'.format(
+                get_mosfit_hash()))
+        hks_loaded = False
+        if os.path.isfile(key_cache_path):
+            try:
+                self._header_keys = pickle.load(open(key_cache_path, 'rb'))
+                hks_loaded = True
+            except Exception:
+                printer.message('bad_header_pickle', warning=True)
+                if hasattr(self, '_header_keys'):
+                    del(self._header_keys)
+        if not hks_loaded:
+            self._header_keys = OrderedDict((
+                (PHOTOMETRY.TIME, [
+                    'time', 'mjd', ('jd', 'jd'), ('julian date', 'jd'),
+                    ('date', 'yyyy-mm-dd'), 'day', (
+                        'kiloseconds', 'kiloseconds')]),
+                (PHOTOMETRY.SYSTEM, ['system', 'magsys', 'magnitude system']),
+                (PHOTOMETRY.MAGNITUDE, [
+                 'vega mag', 'ab mag', 'mag', 'magnitude']),
+                (PHOTOMETRY.E_MAGNITUDE, self._emagstrs),
+                (PHOTOMETRY.TELESCOPE, ['tel', 'telescope']),
+                (PHOTOMETRY.INSTRUMENT, ['inst', 'instrument']),
+                (PHOTOMETRY.OBSERVER, ['observer']),
+                (PHOTOMETRY.OBSERVATORY, ['observatory']),
+                (PHOTOMETRY.BAND, ['passband', 'band', 'filter', 'filt',
+                                   'flt']),
+                (PHOTOMETRY.E_LOWER_MAGNITUDE, [a + ' ' + b for a, b in chain(
+                    product(self._emagstrs, ['minus', 'lower']),
+                    product(['minus', 'lower'], self._emagstrs))]),
+                (PHOTOMETRY.E_UPPER_MAGNITUDE, [a + ' ' + b for a, b in chain(
+                    product(self._emagstrs, ['plus', 'upper']),
+                    product(['plus', 'upper'], self._emagstrs))]),
+                (PHOTOMETRY.UPPER_LIMIT, [
+                    'upper limit', 'upperlimit', 'l_mag', 'limit']),
+                (PHOTOMETRY.COUNT_RATE, [
+                    'count', 'counts', 'flux', 'count rate']),
+                (PHOTOMETRY.E_COUNT_RATE, self._ecntstrs),
+                (PHOTOMETRY.FLUX_DENSITY, ['flux density', 'fd', 'f_nu']),
+                (PHOTOMETRY.E_FLUX_DENSITY, [
+                    'e_flux_density', 'flux density error', 'e_fd',
+                    'sigma_nu']),
+                (PHOTOMETRY.U_FLUX_DENSITY, []),
+                (PHOTOMETRY.ZERO_POINT, ['zero point', 'zp']),
+                ('reference', ['reference', 'bibcode', 'source', 'origin']),
+                (ENTRY.NAME, [
+                    'event', 'transient', 'name', 'supernova', 'sne', 'id',
+                    'identifier', 'object']),
+                (ENTRY.REDSHIFT, ['redshift']),
+                (ENTRY.LUM_DIST, [
+                    'lumdist', 'luminosity distance', 'distance']),
+                (ENTRY.COMOVING_DIST, ['comoving distance']),
+                (ENTRY.RA, ['ra', 'right ascension', 'right_ascension']),
+                (ENTRY.DEC, ['dec', 'declination']),
+                (ENTRY.EBV, ['ebv', 'extinction']),
+                # At the moment transient-specific keys are not in astrocats.
+                ('claimedtype', [
+                    'claimedtype', 'type', 'claimed_type', 'claimed type'])
+            ))
+
         self._critical_keys = [
             PHOTOMETRY.TIME, PHOTOMETRY.MAGNITUDE, PHOTOMETRY.COUNT_RATE,
             PHOTOMETRY.FLUX_DENSITY,
@@ -134,21 +181,26 @@ class Converter(object):
         self._bool_keys = [PHOTOMETRY.UPPER_LIMIT]
         self._specify_keys = [
             PHOTOMETRY.BAND, PHOTOMETRY.INSTRUMENT, PHOTOMETRY.TELESCOPE]
+        self._entry_keys = [
+            ENTRY.COMOVING_DIST, ENTRY.REDSHIFT, ENTRY.LUM_DIST,
+            ENTRY.RA, ENTRY.DEC, ENTRY.EBV, 'claimedtype']
         self._use_mc = False
         self._month_rep = re.compile(
             r'\b(' + '|'.join(self._MONTH_IDS.keys()) + r')\b')
         self._converted = []
 
-        for key in self._header_keys.keys():
-            for val in self._header_keys[key]:
-                for i in range(val.count(' ')):
-                    rep = val.replace(' ', '_', i + 1)
-                    if rep not in self._header_keys[key]:
-                        self._header_keys[key].append(rep)
-                for i in range(val.count(' ')):
-                    rep = val.replace(' ', '', i + 1)
-                    if rep not in self._header_keys[key]:
-                        self._header_keys[key].append(rep)
+        if not hks_loaded:
+            for key in self._header_keys.keys():
+                for val in self._header_keys[key]:
+                    for i in range(val.count(' ')):
+                        rep = val.replace(' ', '_', i + 1)
+                        if rep not in self._header_keys[key]:
+                            self._header_keys[key].append(rep)
+                    for i in range(val.count(' ')):
+                        rep = val.replace(' ', '', i + 1)
+                        if rep not in self._header_keys[key]:
+                            self._header_keys[key].append(rep)
+            pickle.dump(self._header_keys, open(key_cache_path, 'wb'))
 
     def generate_event_list(self, event_list):
         """Generate a list of events and/or convert events to JSON format."""
@@ -205,7 +257,12 @@ class Converter(object):
                         'Space: ` `', 'Tab: `\t`', 'Comma: `,`',
                         'Semi-colon: `;`', 'Bar: `|`', 'Ampersand: `&`']
                     delim = None
-                    delimcounts = [ftxt.count(x) for x in delims]
+                    delimcounts = [re.sub(
+                        re.escape(y) + '+', y, re.sub(
+                            ' ?[' + ''.join([re.escape(
+                                x) for x in delims if x != y]) + ']' +
+                            ' ?', '', ftxt)).count(
+                                y) for y in delims]
                     maxdelimcount = max(delimcounts)
                     delim = delims[delimcounts.index(maxdelimcount)]
                     # If two delimiter options are close in count, ask user.
@@ -222,9 +279,27 @@ class Converter(object):
                     ad = ''.join(ad)
 
                     fsplit = ftxt.splitlines()
+
+                    # See if we need to append blank errors to upper limits.
+                    tsplit = [
+                        replace_multiple(x, ['$', '\\pm', '±', '-or+'], delim)
+                        .strip(ad + '()# ').replace('′', "'")
+                        for x in fsplit]
+
+                    append_missing_errs = False
+                    for fl in tsplit:
+                        dfl = list(csv.reader([fl], delimiter=delim))[0]
+                        if any([is_number(x.strip('(<>≤≥'))
+                                for x in dfl]) and any([
+                                any([y in x for y in [
+                                    '(', '<', '>', '≥', '≤']])
+                                for x in dfl]):
+                            append_missing_errs = True
+                            break
+
                     fsplit = [
-                        x.replace('$', '').replace('\\pm', delim)
-                        .replace('±', delim).replace('(', delim + '(')
+                        replace_multiple(x, ['$', '\\pm', '±', '-or+'], delim)
+                        .replace('(', delim + '(')
                         .strip(ad + '()# ').replace('′', "'")
                         for x in fsplit]
                     flines = []
@@ -232,6 +307,8 @@ class Converter(object):
                         # Replace repeated spaces if fixed-width
                         if delim in [' ']:
                             fsn = re.sub(r'(\s)\1+', r'\1', fs)
+                        else:
+                            fsn = fs
                         flines.append(list(
                             csv.reader([fsn], delimiter=delim))[0])
 
@@ -244,15 +321,29 @@ class Converter(object):
                     for fi, fl in enumerate(list(flines)):
                         flcopy = list(fl)
                         offset = 0
-                        if not any([is_number(x) for x in fl]):
+                        if not any([is_datum(x) for x in fl]):
                             for fci, fc in enumerate(fl):
                                 if (fc in self._band_names and
                                     (fci == len(fl) - 1 or
                                      fl[fci + 1] not in self._emagstrs)):
-                                    flcopy.insert(fci + 1 + offset, 'e mag')
+                                    flcopy.insert(fci + 1 + offset, 'e_' + fc)
                                     offset += 1
                         flines[fi] = flcopy
 
+                    # Append blank errors to upper limits.
+                    if append_missing_errs:
+                        # Find the most frequent column count. These are
+                        # probably the tables we wish to read.
+                        flens = [len(x) for x in flines]
+                        ncols = Counter(flens).most_common(1)[0][0]
+
+                        flines = [[x for y in [
+                            ([z, '-'] if (any([w in z for w in [
+                                '<', '>', '≤', '≥']]) or
+                                z in self._EMPTY_VALS) else [z])
+                            for z in fl] for x in y] if len(
+                                fl) != ncols else fl for fl in flines]
+
                     # Find the most frequent column count. These are probably
                     # the tables we wish to read.
                     flens = [len(x) for x in flines]
@@ -267,20 +358,20 @@ class Converter(object):
                             potential_name = fl[0]
                         if flens[fi] == ncols:
                             if potential_name is not None and any(
-                                    [is_number(x) for x in fl]):
+                                    [is_datum(x) for x in fl]):
                                 newlines.append([potential_name] + list(fl))
                             else:
                                 newlines.append(list(fl))
                     flines = newlines
                     for fi, fl in enumerate(flines):
                         if len(fl) == ncols and potential_name is not None:
-                            if not any([is_number(x) for x in fl]):
+                            if not any([is_datum(x) for x in fl]):
                                 flines[fi] = ['name'] + list(fl)
 
                 # If none of the rows contain numeric data, the file
                 # is likely a list of transient names.
                 if (len(flines) and
-                    (not any(any([is_number(x) or x == '' for x in y])
+                    (not any(any([is_datum(x) or x == '' for x in y])
                              for y in flines) or
                      len(flines) == 1)):
                     new_events = [
@@ -289,7 +380,7 @@ class Converter(object):
                 # If last row is numeric, then likely this is a file with
                 # transient data.
                 elif (len(flines) > 1 and
-                        any([is_number(x) for x in flines[-1]])):
+                        any([is_datum(x) for x in flines[-1]])):
 
                     # Check that each row has the same number of columns.
                     if len(set([len(x) for x in flines])) > 1:
@@ -400,27 +491,42 @@ class Converter(object):
                              len(cidict[PHOTOMETRY.TIME]) <= 2)):
                         bi = cidict[PHOTOMETRY.TIME]
 
+                        bistr = bi[0].upper() if isinstance(
+                            bi, list) and not isinstance(
+                                bi, string_types) else 'MJD'
+
                         if isinstance(bi, list) and not isinstance(
                             bi, string_types) and isinstance(
-                                bi[0], string_types) and bi[0] == 'jd':
+                                bi[0], string_types):
                             bi = bi[-1]
 
-                        mmtimes = [float(x[bi])
-                                   for x in flines[self._first_data:]]
-                        mintime, maxtime = min(mmtimes), max(mmtimes)
+                        mmtimes = None
+                        try:
+                            mmtimes = [float(x[bi])
+                                       for x in flines[self._first_data:]]
+                            mintime, maxtime = min(mmtimes), max(mmtimes)
+                        except Exception:
+                            pass
 
-                        if mintime < 10000:
+                        if not mmtimes:
+                            pass
+                        elif (bistr == 'MJD' and mintime < 10000 or
+                              bistr == 'JD' and mintime < 2410000 or
+                                bistr in ['KILOSECONDS', 'SECONDS']):
                             while True:
+                                pstr = ('s_offset' if bistr in [
+                                    'KILOSECONDS', 'SECONDS'] else
+                                    'small_time_offset')
                                 try:
-                                    response = prt.prompt(
-                                        'small_time_offset', kind='string')
+                                    response = prt.prompt(pstr, [
+                                        bistr for x in range(3)],
+                                        kind='string')
                                     if response is not None:
                                         toffset = Decimal(response)
                                     break
                                 except Exception:
                                     pass
-                        elif maxtime > 60000 and cidict[
-                                PHOTOMETRY.TIME][0] != 'jd':
+                        elif maxtime > 90000 and bistr != 'JD':
                             isjd = prt.prompt(
                                 'large_time_offset',
                                 kind='bool', default='y')
@@ -459,14 +565,19 @@ class Converter(object):
                                     if rval:
                                         row[cidict[key]] = rval
                                 elif key == 'reference':
-                                    if (isinstance(cidict[key],
-                                                   string_types) and
-                                            len(cidict[key]) == 19):
+                                    if (isinstance(row[cidict[key]],
+                                                   string_types)):
+                                        srcdict = OrderedDict()
+                                        if is_bibcode(row[cidict[key]]):
+                                            srcdict[SOURCE.BIBCODE] = row[
+                                                cidict[key]]
+                                        else:
+                                            srcdict[SOURCE.NAME] = row[
+                                                cidict[key]]
                                         new_src = entries[rname].add_source(
-                                            bibcode=cidict[key])
+                                            **srcdict)
                                         sources.update(new_src)
-                                        row[
-                                            cidict[key]] = new_src
+                                        # row[cidict[key]] = new_src
                                 elif key == ENTRY.NAME:
                                     continue
                                 elif (isinstance(key, Key) and
@@ -484,9 +595,19 @@ class Converter(object):
                                                 x.group()], date)
                                         photodict[key] = str(
                                             astrotime(date, format='isot').mjd)
-                                    elif cidict[key][0] == 'jd':
+                                    elif cidict[key][0].upper() == 'JD':
                                         photodict[key] = str(
                                             jd_to_mjd(Decimal(tval[-1])))
+                                    elif cidict[key][0].upper(
+                                    ) == 'KILOSECONDS':
+                                        photodict[key] = str(Decimal(
+                                            KS_DAYS) * Decimal(tval[-1]))
+                                    elif cidict[key][0].upper(
+                                    ) == 'YYYY-MM-DD':
+                                        photodict[key] = str(
+                                            astrotime(
+                                                tval[-1].replace('/', '-'),
+                                                format='isot').mjd)
                                     continue
 
                                 val = cidict[key]
@@ -601,10 +722,7 @@ class Converter(object):
                                                 allow_blank=False
                                             )
                                             bibcode = bibcode.strip()
-                                            if (re.search(
-                                                '[0-9]{4}..........[\.0-9]{4}'
-                                                '[A-Za-z]', bibcode)
-                                                    is None):
+                                            if not is_bibcode(bibcode):
                                                 bibcode = ''
                                         rsource[
                                             SOURCE.BIBCODE] = bibcode
@@ -639,21 +757,21 @@ class Converter(object):
 
                                     shared_sources.append(rsource)
 
-                            if len(shared_sources):
-                                src_list = []
+                            if len(sources) or len(shared_sources):
+                                src_list = list(sources)
                                 for src in shared_sources:
                                     src_list.append(entries[
                                         rname].add_source(**src))
-                                    photodict[PHOTOMETRY.SOURCE] = ','.join(
-                                        src_list)
+                                photodict[PHOTOMETRY.SOURCE] = ','.join(
+                                    src_list)
 
                             if any([x in photodict.get(
                                     PHOTOMETRY.MAGNITUDE, '')
-                                    for x in ['<', '>']]):
+                                    for x in ['<', '>', '≤', '≥']]):
                                 photodict[PHOTOMETRY.UPPER_LIMIT] = True
                                 photodict[
                                     PHOTOMETRY.MAGNITUDE] = photodict[
-                                        PHOTOMETRY.MAGNITUDE].strip('<>')
+                                        PHOTOMETRY.MAGNITUDE].strip('<>≤≥')
 
                             if '<' in photodict.get(PHOTOMETRY.COUNT_RATE, ''):
                                 photodict[PHOTOMETRY.UPPER_LIMIT] = True
@@ -717,6 +835,18 @@ class Converter(object):
                             entries[rname].add_photometry(
                                 **photodict)
 
+                            # Add other entry keys.
+                            for key in self._entry_keys:
+                                if key not in cidict:
+                                    continue
+                                qdict = OrderedDict()
+                                qdict[QUANTITY.VALUE] = row[
+                                    cidict[key]]
+                                if len(sources) or len(shared_sources):
+                                    qdict[QUANTITY.SOURCE] = ','.join(
+                                        src_list)
+                                entries[rname].add_quantity(key, **qdict)
+
                     merge_with_existing = None
                     for ei, entry in enumerate(entries):
                         entries[entry].sanitize()
@@ -758,9 +888,10 @@ class Converter(object):
         akeys = list(self._critical_keys) + list(self._helpful_keys)
         dkeys = list(self._dep_keys)
         prt = self._printer
+        mpatt = re.compile('mag', re.IGNORECASE)
 
         for fi, fl in enumerate(flines):
-            if not any([is_number(x) for x in fl]):
+            if not any([is_datum(x) for x in fl]):
                 # Try to associate column names with common header keys.
                 conflict_keys = []
                 conflict_cis = []
@@ -812,16 +943,17 @@ class Converter(object):
                 del(cidict[used_cis[ci]])
                 del(used_cis[ci])
             for fi, fl in enumerate(flines):
-                if not any([is_number(x) for x in fl]):
+                if not any([is_datum(x) for x in fl]):
                     # Try to associate column names with common header keys.
                     for ci, col in enumerate(fl):
                         if ci in used_cis:
                             continue
-                        if col in self._band_names:
+                        ccol = mpatt.sub('', col)
+                        if ccol in self._band_names:
                             cidict.setdefault(key, []).append(ci)
                             used_cis[ci] = key
-                            cidict.setdefault(bkey, []).append(col)
-                        elif col in self._emagstrs:
+                            cidict.setdefault(bkey, []).append(ccol)
+                        elif ccol in self._emagstrs:
                             cidict.setdefault(ekey, []).append(ci)
                             used_cis[ci] = ekey
 
@@ -847,26 +979,17 @@ class Converter(object):
                 'counts_mags_fds', kind='option',
                 options=['Magnitudes', 'Counts (per second)',
                          'Flux Densities (Jansky)'],
-                none_string=None)
-        if self._data_type in [1, 3]:
+                none_string='No Photometry', default='1')
+        if self._data_type in [1, 3, 'n']:
             akeys.remove(PHOTOMETRY.COUNT_RATE)
             akeys.remove(PHOTOMETRY.E_COUNT_RATE)
             akeys.remove(PHOTOMETRY.ZERO_POINT)
-            if (PHOTOMETRY.MAGNITUDE in akeys and
-                    PHOTOMETRY.E_MAGNITUDE in akeys):
-                akeys.remove(PHOTOMETRY.E_MAGNITUDE)
-                akeys.insert(
-                    akeys.index(PHOTOMETRY.MAGNITUDE) + 1,
-                    PHOTOMETRY.E_MAGNITUDE)
-            if (PHOTOMETRY.E_LOWER_MAGNITUDE in cidict and
-                    PHOTOMETRY.E_UPPER_MAGNITUDE in cidict):
-                akeys.remove(PHOTOMETRY.E_MAGNITUDE)
             dkeys.remove(PHOTOMETRY.E_COUNT_RATE)
-        if self._data_type in [2, 3]:
+        if self._data_type in [2, 3, 'n']:
             akeys.remove(PHOTOMETRY.MAGNITUDE)
             akeys.remove(PHOTOMETRY.E_MAGNITUDE)
             dkeys.remove(PHOTOMETRY.E_MAGNITUDE)
-        if self._data_type in [1, 2]:
+        if self._data_type in [1, 2, 'n']:
             akeys.remove(PHOTOMETRY.FLUX_DENSITY)
             akeys.remove(PHOTOMETRY.E_FLUX_DENSITY)
             if (PHOTOMETRY.E_LOWER_FLUX_DENSITY in cidict and
@@ -874,6 +997,23 @@ class Converter(object):
                 akeys.remove(PHOTOMETRY.E_FLUX_DENSITY)
             dkeys.remove(PHOTOMETRY.E_FLUX_DENSITY)
             dkeys.remove(PHOTOMETRY.U_FLUX_DENSITY)
+        if self._data_type == 'n':
+            akeys.remove(PHOTOMETRY.TIME)
+            akeys.remove(PHOTOMETRY.BAND)
+            akeys.remove(PHOTOMETRY.INSTRUMENT)
+            akeys.remove(PHOTOMETRY.TELESCOPE)
+
+        # Make sure `E_` keys always appear after the actual measurements.
+        if (PHOTOMETRY.MAGNITUDE in akeys and
+                PHOTOMETRY.E_MAGNITUDE in akeys):
+            akeys.remove(PHOTOMETRY.E_MAGNITUDE)
+            akeys.insert(
+                akeys.index(PHOTOMETRY.MAGNITUDE) + 1,
+                PHOTOMETRY.E_MAGNITUDE)
+        # Remove regular `E_` keys if both `E_LOWER_`/`E_UPPER_` exist.
+        if (PHOTOMETRY.E_LOWER_MAGNITUDE in cidict and
+                PHOTOMETRY.E_UPPER_MAGNITUDE in cidict):
+            akeys.remove(PHOTOMETRY.E_MAGNITUDE)
 
         columns = np.array(flines[self._first_data:]).T.tolist()
         colstrs = np.array([
@@ -895,16 +1035,16 @@ class Converter(object):
                 continue
             if key.type == KEY_TYPES.NUMERIC:
                 lcolinds = [x for x in colinds
-                            if any(is_number(y) for y in columns[x]) and
+                            if any(is_datum(y) for y in columns[x]) and
                             x not in selected_cols]
             elif key.type == KEY_TYPES.TIME:
                 lcolinds = [x for x in colinds
-                            if any(is_date(y) or is_number(y)
+                            if any(is_date(y) or is_datum(y)
                                    for y in columns[x]) and
                             x not in selected_cols]
             elif key.type == KEY_TYPES.STRING:
                 lcolinds = [x for x in colinds
-                            if any(not is_number(y) for y in columns[x]) and
+                            if any(not is_datum(y) for y in columns[x]) and
                             x not in selected_cols]
             else:
                 lcolinds = [x for x in colinds if x not in selected_cols]
@@ -1053,7 +1193,8 @@ class Converter(object):
                 self._system = systems[int(self._system) - 1]
 
         if (PHOTOMETRY.INSTRUMENT not in cidict and
-                PHOTOMETRY.TELESCOPE not in cidict):
+            PHOTOMETRY.TELESCOPE not in cidict and
+                PHOTOMETRY.MAGNITUDE in cidict):
             prt.message('instrument_recommended', warning=True)
 
     def get_converted(self):
