@@ -97,13 +97,15 @@ class Model(object):
 
             all_models_txt = prt.text('all_models')
             suggested_models_txt = prt.text('suggested_models', [claimed_type])
+            another_model_txt = prt.text('another_model')
 
-            if claimed_type is None:
+            type_options = model_types.get(
+                claimed_type, []) if claimed_type else []
+            if not type_options:
                 type_options = all_models
                 model_prompt_txt = all_models_txt
             else:
-                type_options = model_types.get(claimed_type, []) + [
-                    'Another model not listed here.']
+                type_options.append(another_model_txt)
                 model_prompt_txt = suggested_models_txt
             if not type_options:
                 prt.message('no_model_for_type', warning=True)
@@ -121,10 +123,10 @@ class Model(object):
                             none_string=('None of the above, skip this '
                                          'transient.'))
                         if sel is not None:
-                            self._model_name = type_options[sel]
+                            self._model_name = type_options[sel - 1]
                     if not self._model_name:
                         break
-                    if self._model_name == 'Another model not listed here.':
+                    if self._model_name == another_model_txt:
                         type_options = all_models
                         model_prompt_txt = all_models_txt
                         self._model_name = None
@@ -216,6 +218,7 @@ class Model(object):
         self._modules = OrderedDict()
         self._bands = []
         self._instruments = []
+        self._telescopes = []
 
         # Load the call tree for the model. Work our way in reverse from the
         # observables, first constructing a tree for each observable and then
@@ -280,6 +283,7 @@ class Model(object):
                 cur_task.update(self._parameter_json[task])
             self._modules[task] = self._load_task_module(task)
             if mod_name == 'photometry':
+                self._telescopes = self._modules[task].telescopes()
                 self._instruments = self._modules[task].instruments()
                 self._bands = self._modules[task].bands()
             self._modules[task].set_attributes(cur_task)
@@ -402,7 +406,8 @@ class Model(object):
                     data,
                     req_key_values=OrderedDict((
                         ('band', self._bands),
-                        ('instrument', self._instruments))),
+                        ('instrument', self._instruments),
+                        ('telescope', self._telescopes))),
                     subtract_minimum_keys=['times'],
                     smooth_times=smooth_times,
                     extrapolate_time=extrapolate_time,
@@ -496,7 +501,7 @@ class Model(object):
                                           s[0] != 'AB') else '')))) +
                 ']').replace(' []', '') for s in list(sorted(filterarr))]
             if not all(ois):
-                filterrows.append('  (* = Not observed in this band)')
+                filterrows.append(prt.text('not_observed'))
             prt.prt('\n'.join(filterrows))
 
             single_freq_inst = list(
